/*
 * Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * This file is part of the FSG PAPI Project
 *
 * The FSG PAPI Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FSG PAPI Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with the FSG PAPI Project; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: lprm.c,v 1.1.1.1 2002/09/04 20:57:34 jacobs Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <libintl.h>
#include <papi.h>

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout, "Usage: %s [-P printer] (user|id ...)\n", name);
	exit (1);
}

int
main(int ac, char *av[])
{
	int exit_code = 0;
	char *printer = NULL;
	char *password = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	int c;

	while ((c = getopt(ac, av, "P:")) != EOF)
		switch (c) {
		case 'P':
			printer = optarg;
			break;
		case 'p':
			password = optarg;
			break;
		default:
			usage(av[0]);
		}

	if ((printer == NULL) &&
	    ((printer = getenv("PRINTER")) == NULL) &&
	    ((printer = getenv("LPDEST")) == NULL))
		printer = "lp";

	for (c = optind; c < ac; c++) {
		papi_status_t status;
		papi_service_t svc = NULL;
		char *user = NULL;
		char *value = av[c];
		int id = -1;

		if (isdigit(value[0]) != 0)
			id = atoi(value);
		else
			user = value; 

		status = papiServiceCreate(&svc, value, user, password, NULL,
					encryption, NULL);
		if (status != PAPI_OK) {
			char *mesg = papiServiceGetStatusMessage(svc);

			if (mesg == NULL)
				mesg = papiStatusString(status);

			fprintf(stderr, gettext(
				"Failed to contact service for %s: %s\n"),
				value, mesg);
			exit(1);
		}

		if (id != -1) { /* is an individual job */
			char *mesg;

			status = papiJobCancel(svc, printer, id);
			if (status != PAPI_OK) {
				mesg = papiServiceGetStatusMessage(svc);
				if (mesg == NULL)
					mesg = papiStatusString(status);
				exit_code = 1;
			} else
				mesg = "cancelled";
			fprintf(stderr, "%s-%d: %s\n", printer, id, mesg);
		} else { /* is a user */
			papi_job_t *jobs = NULL;
			char *mesg;

			status = papiPrinterPurgeJobs(svc, printer, &jobs);
			switch (status) {
			case PAPI_OK_SUBST:
				printf(gettext("%s: queue empty\n"), printer);
				break;
			case PAPI_OK: {
				int i;

				for (i = 0; jobs[i] != NULL; i++) {
					int id;

					id = papiJobGetID(jobs[i]);
					mesg = "cancelled";
					printf("%d: %s\n", id, mesg);
				}}
				break;
			default:
				mesg = papiServiceGetStatusMessage(svc);
				if (mesg == NULL)
					mesg = papiStatusString(status);
				exit_code = 1;
				fprintf(stderr, gettext("PurgeJobs %s: %s\n"),
					printer, mesg);
			}
			papiJobListFree(jobs);
		}
	
		papiServiceDestroy(svc);
	}

	exit(exit_code);
}
