/*
 * Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * This file is part of the FSG PAPI Project
 *
 * The FSG PAPI Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FSG PAPI Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with the FSG PAPI Project; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: job.c,v 1.2 2002/10/24 06:13:24 jacobs Exp $ */
#pragma ident	"@(#)job.c	1.1	02/08/22 SMI"

/*LINTLIBRARY*/

#include <stdlib.h>
#include <papi_impl.h>

void
papiJobFree(papi_job_t job)
{
	job_t *tmp = (job_t *)job;

	if (tmp != NULL) {
		papiAttributeListFree(tmp->attributes);
	}
}

void
papiJobListFree(papi_job_t *jobs)
{
	job_t **tmp = (job_t **)jobs;

	if (tmp != NULL) {
		int i;

		for (i = 0 ; tmp[i] != NULL ; i++)
			free(tmp[i]);
		free(tmp);
	}
}

papi_attribute_t **
papiJobGetAttributeList(papi_job_t job)
{
	job_t *tmp = (job_t *)job;

	if (tmp != NULL)
		return (tmp->attributes);

	return (NULL);
}

char *
papiJobGetPrinterName(papi_job_t job)
{
	job_t *tmp = (job_t *)job;
	char *result = NULL;

	if (tmp != NULL)
		papiAttributeListGetString(tmp->attributes, NULL,
					"printer-name", &result);

	return (result);
}

int32_t
papiJobGetID(papi_job_t job)
{
	job_t *tmp = (job_t *)job;
	int result = -1;

	if (tmp != NULL)
		papiAttributeListGetInteger(tmp->attributes, NULL, "job-id",
                                        &result);

	return (result);
}

papi_job_ticket_t *
papiJobGetJobTicket(papi_job_t job)
{
	return (NULL);	/* NOT SUPPORTED */
}

papi_status_t
papiJobSubmit(papi_service_t handle, const char *printer,
		const papi_attribute_t **job_attributes,
		const papi_job_ticket_t *job_ticket,
		const char **files, papi_job_t *job)
{
	service_t *svc = handle;
	job_t *j = NULL;

	if ((svc == NULL) || (printer == NULL) || (files == NULL) ||
	    (job == NULL))
		return (PAPI_BAD_ARGUMENT);

	if (job_ticket != NULL)
		return (PAPI_OPERATION_NOT_SUPPORTED);

	if ((*job = j = calloc(1, sizeof (*j))) == NULL)
		return (PAPI_TEMPORARY_ERROR);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}

papi_status_t
papiJobSubmitByReference(papi_service_t handle, const char *printer,
		const papi_attribute_t **job_attributes,
		const papi_job_ticket_t *job_ticket,
		const char **files, papi_job_t *job)
{
	service_t *svc = handle;
	job_t *j = NULL;

	if ((svc == NULL) || (printer == NULL) || (files == NULL) ||
	    (job == NULL))
		return (PAPI_BAD_ARGUMENT);

	if (job_ticket != NULL)
		return (PAPI_OPERATION_NOT_SUPPORTED);

	if ((*job = j = calloc(1, sizeof (*j))) == NULL)
		return (PAPI_TEMPORARY_ERROR);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}

papi_status_t
papiJobValidate(papi_service_t handle, const char *printer,
		const papi_attribute_t **job_attributes,
		const papi_job_ticket_t *job_ticket,
		const char **files, papi_job_t *job)
{
	papi_status_t status;
	papi_attribute_t **attributes = NULL;
	int i;

	papiAttributeListAddString(&attributes, PAPI_ATTR_REPLACE,
			"job-hold-until", "indefinite");
	for (i = 0; job_attributes[i]; i++)
		list_append(&attributes, job_attributes[i]);

	status = papiJobSubmit(handle, printer,
				(const papi_attribute_t **)attributes,
				job_ticket, files, job);
	if (status == PAPI_OK) {
		int id = papiJobGetID(*job);

		papiJobCancel(handle, printer, id);
	}

	return (status);
}

papi_status_t papiJobStreamOpen(papi_service_t handle, const char *printer,
		const papi_attribute_t **job_attributes,
		const papi_job_ticket_t *job_ticket, papi_stream_t *stream)
{
	service_t *svc = handle;
	job_stream_t *s = NULL;

	if ((svc == NULL) || (printer == NULL) || (stream == NULL))
		return (PAPI_BAD_ARGUMENT);

	if (job_ticket != NULL)
		return (PAPI_OPERATION_NOT_SUPPORTED);

	if ((*stream = s = calloc(1, sizeof (*s))) == NULL)
		return (PAPI_TEMPORARY_ERROR);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}

extern papi_status_t papiJobStreamWrite(papi_service_t handle,
		papi_stream_t stream, const void *buffer, const size_t buflen)
{
	service_t *svc = handle;
	job_stream_t *s = stream;

	if ((svc == NULL) || (stream == NULL) || (buffer == NULL) ||
	    (buflen < 0))
		return (PAPI_BAD_ARGUMENT);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}
extern papi_status_t papiJobStreamClose(papi_service_t handle,
		papi_stream_t stream, papi_job_t *job)
{
	service_t *svc = handle;
	job_stream_t *s = stream;
	job_t *j = NULL;

	if ((svc == NULL) || (stream == NULL) || (job == NULL))
		return (PAPI_BAD_ARGUMENT);

	if ((*job = j = calloc(1, sizeof (*j))) == NULL)
		return (PAPI_TEMPORARY_ERROR);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}


papi_status_t
papiJobQuery(papi_service_t handle, const char *printer, const int32_t job_id,
		const char **requested_attrs,
		papi_job_t *job)
{
	service_t *svc = handle;

	if ((svc == NULL) || (printer == NULL))
		return (PAPI_BAD_ARGUMENT);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}

papi_status_t
papiJobCancel(papi_service_t handle, const char *printer, const int32_t job_id)
{
	service_t *svc = handle;

	if ((svc == NULL) || (printer == NULL) || (job_id < 0))
		return (PAPI_BAD_ARGUMENT);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}

papi_status_t
papiJobHold(papi_service_t handle, const char *printer, const int32_t job_id,
		const char *hold_until, const time_t *hold_until_time)
{
	service_t *svc = handle;

	if ((svc == NULL) || (printer == NULL) || (job_id < 0))
		return (PAPI_BAD_ARGUMENT);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}

papi_status_t
papiJobRelease(papi_service_t handle, const char *printer, const int32_t job_id)
{
	service_t *svc = handle;

	if ((svc == NULL) || (printer == NULL) || (job_id < 0))
		return (PAPI_BAD_ARGUMENT);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}

papi_status_t
papiJobRestart(papi_service_t handle, const char *printer, const int32_t job_id)
{
	service_t *svc = handle;

	if ((svc == NULL) || (printer == NULL) || (job_id < 0))
		return (PAPI_BAD_ARGUMENT);

	return (PAPI_OPERATION_NOT_SUPPORTED);
}

papi_status_t
papiJobModify(papi_service_t handle, const char *printer, const int32_t job_id,
		const papi_attribute_t **attributes, papi_job_t *result)
{
	service_t *svc = handle;

	if ((svc == NULL) || (printer == NULL) || (job_id < 0) ||
	    (attributes == NULL))
		return (PAPI_BAD_ARGUMENT);

 	return (PAPI_OPERATION_NOT_SUPPORTED);
}
