/*
 * Copyright (C) 2002 by Syn Microsystems, Inc.  All Rights Reserved.
 * Use is subject to license terms
 */

/* $Id: attribute.c,v 1.1.1.1 2002/09/04 20:57:35 jacobs Exp $ */

#include <stdio.h>
#include <papi.h>

timet_strftime(char *buf, size_t size, char *fmt, time_t *c)
{
	struct tm *tm = gmtime(c);
	strftime(buf, size, fmt, tm);
}

test_collection()
{
	int i;
	papi_status_t status;
	papi_attribute_t **list = NULL,
			 **collection = NULL,
			 **col2 = NULL;
	char *findme = NULL;

	for (i = 0; i < 3; i++) {
		char name[12], value[12];

		snprintf(name, sizeof (name), "string-%d", i);
		snprintf(value, sizeof (value), "value-%d", i);
		papiAttributeListAddString(&list, PAPI_ATTR_APPEND,
					name, value);
	}

	for (i = 0; i < 3; i++) {
		char name[12], value[12];

		snprintf(name, sizeof (name), "string-%d", i);
		snprintf(value, sizeof (value), "value-%d", i);
		papiAttributeListAddString(&col2, PAPI_ATTR_APPEND,
					name, value);
	}

	for (i = 0; i < 5; i++) {
		char name[32];
		papi_attribute_t **col = NULL;
		int j;

		for (j = 0; j < 5; j++) {
			char value[12];

			snprintf(name, sizeof (name), "string-%d-%d", i, j);
			snprintf(value, sizeof (value), "value-%d", j);
			papiAttributeListAddString(&col, PAPI_ATTR_APPEND,
					name, value);
			if (i == j) {
				snprintf(name, sizeof (name), "col-%d-%d", i, j);
				status = papiAttributeListAddCollection(&col,
					PAPI_ATTR_APPEND, name, col2);
				printf("papiAttributeListAddCollection(&col, PAPI_ATTR_APPEND, %s, col2): %s\n", name,
					papiStatusString(status));
			}
		}
		snprintf(name, sizeof (name), "collection-%d", i);
		findme = strdup(name);
		status = papiAttributeListAddCollection(&list, PAPI_ATTR_APPEND,
				name, col);
		printf("papiAttributeListAddCollection(&list, PAPI_ATTR_APPEND, %s, col): %s\n", name,
					papiStatusString(status));
		status = papiAttributeListAddCollection(&list, PAPI_ATTR_EXCL,
				name, col);
		printf("papiAttributeListAddCollection(&list, PAPI_ATTR_EXCL, %s, col): %s\n", name,
					papiStatusString(status));

		papiAttributeListFree(col);
	}

	for (i = 0; i < 3; i++) {
		char name[12];

		snprintf(name, sizeof (name), "int-%d", i);
		papiAttributeListAddInteger(&list, PAPI_ATTR_APPEND, name, i);
	}

	status = papiAttributeListGetCollection(list, NULL, findme, &col2);
	printf("papiAttributeListGetCollection(list, NULL, %s, &col2): %s (0x%8.8x)\n", findme,
					papiStatusString(status), col2);

	papiAttributeListPrint(stdout, "COLLECTION:\t", list);
	papiAttributeListFree(list);
}


test_string()
{
	papi_status_t status;
	papi_attribute_t **list = NULL;
	void *iterator = NULL;
	char *value = NULL;

	status = papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"string-1", "value-1");
	printf("papiAttributeListAddString(&list, PAPI_ATTR_EXCL, \"string-1\", \"value-1\"): %s\n", papiStatusString(status));

	status = papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"string-1", "value-2");
	printf("papiAttributeListAddString(&list, PAPI_ATTR_EXCL, \"string-1\", \"value-2\"): %s\n", papiStatusString(status));

	status = papiAttributeListAddString(&list, PAPI_ATTR_APPEND,
					"string-1", "value-3");
	printf("papiAttributeListAddString(&list, PAPI_ATTR_APPEND, \"string-1\", \"value-3\"): %s\n", papiStatusString(status));

	status = papiAttributeListGetString(list, &iterator, "string-1",
					&value);
	printf("papiAttributeListGetString(list, &iterator, \"string-1\", &value): %s (%s)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : "NULL"));

	status = papiAttributeListGetString(list, &iterator, NULL, &value);
	printf("papiAttributeListGetString(list, &iterator, NULL, &value): %s (%s)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : "NULL"));

	status = papiAttributeListAddString(&list, PAPI_ATTR_REPLACE,
					"string-1", "value-4");
	printf("papiAttributeListAddString(&list, PAPI_ATTR_REPLACE, \"string-1\", \"value-4\"): %s\n", papiStatusString(status));

	status = papiAttributeListAddString(&list, PAPI_ATTR_REPLACE,
					"string-2", "value-5");
	printf("papiAttributeListAddString(&list, PAPI_ATTR_REPLACE, \"string-2\", \"value-5\"): %s\n", papiStatusString(status));

	status = papiAttributeListGetString(list, &iterator, "string-2",
					&value);
	printf("papiAttributeListGetString(list, &iterator, \"string-2\", &value): %s (%s)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : "NULL"));

	status = papiAttributeListAddString(NULL, PAPI_ATTR_EXCL,
					"string-3", "value-1");
	printf("papiAttributeListAddString(NULL, PAPI_ATTR_EXCL, \"string-3\", \"value-1\"): %s\n", papiStatusString(status));

	status = papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					NULL, "value-1");
	printf("papiAttributeListAddString(&list, PAPI_ATTR_EXCL, NULL, \"value-1\"): %s\n", papiStatusString(status));

	papiAttributeListFree(list);
}

test_integer()
{
	papi_status_t status;
	papi_attribute_t **list = NULL;
	void *iterator = NULL;
	int value = NULL;

	status = papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL,
					"integer-1", 1);
	printf("papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL, \"integer-1\", 1): %s\n", papiStatusString(status));

	status = papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL,
					"integer-1", 2);
	printf("papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL, \"integer-1\", 2): %s\n", papiStatusString(status));

	status = papiAttributeListAddInteger(&list, PAPI_ATTR_APPEND,
					"integer-1", 3);
	printf("papiAttributeListAddInteger(&list, PAPI_ATTR_APPEND, \"integer-1\", 3): %s\n", papiStatusString(status));

	status = papiAttributeListGetInteger(list, &iterator, "integer-1",
					&value);
	printf("papiAttributeListGetInteger(list, &iterator, \"integer-1\", &value): %s (%d)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : -1));

	status = papiAttributeListGetInteger(list, &iterator, NULL, &value);
	printf("papiAttributeListGetInteger(list, &iterator, NULL, &value): %s (%d)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : -1));

	status = papiAttributeListAddInteger(&list, PAPI_ATTR_REPLACE,
					"integer-1", 4);
	printf("papiAttributeListAddInteger(&list, PAPI_ATTR_REPLACE, \"integer-1\", 4): %s\n", papiStatusString(status));

	status = papiAttributeListAddInteger(&list, PAPI_ATTR_REPLACE,
					"integer-2", 5);
	printf("papiAttributeListAddInteger(&list, PAPI_ATTR_REPLACE, \"integer-2\", 5): %s\n", papiStatusString(status));

	status = papiAttributeListGetInteger(list, &iterator, "integer-2",
					&value);
	printf("papiAttributeListGetInteger(list, &iterator, \"integer-2\", &value): %s (%d)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : -1));

	status = papiAttributeListAddInteger(NULL, PAPI_ATTR_EXCL,
					"integer-3", 1);
	printf("papiAttributeListAddInteger(NULL, PAPI_ATTR_EXCL, \"integer-3\", 1): %s\n", papiStatusString(status));

	status = papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL,
					NULL, 1);
	printf("papiAttributeListAddInteger(&list, PAPI_ATTR_EXCL, NULL, 1): %s\n", papiStatusString(status));

	papiAttributeListFree(list);
}

test_boolean()
{
	papi_status_t status;
	papi_attribute_t **list = NULL;
	void *iterator = NULL;
	char value = NULL;

	status = papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL,
					"boolean-1", PAPI_TRUE);
	printf("papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL, \"boolean-1\", PAPI_TRUE): %s\n", papiStatusString(status));

	status = papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL,
					"boolean-1", PAPI_TRUE);
	printf("papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL, \"boolean-1\", PAPI_TRUE): %s\n", papiStatusString(status));

	status = papiAttributeListAddBoolean(&list, PAPI_ATTR_APPEND,
					"boolean-1", PAPI_FALSE);
	printf("papiAttributeListAddBoolean(&list, PAPI_ATTR_APPEND, \"boolean-1\", PAPI_FALSE): %s\n", papiStatusString(status));

	status = papiAttributeListGetBoolean(list, &iterator, "boolean-1",
					&value);
	printf("papiAttributeListGetBoolean(list, &iterator, \"boolean-1\", &value): %s (%d)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : -1));

	status = papiAttributeListGetBoolean(list, &iterator, NULL, &value);
	printf("papiAttributeListGetBoolean(list, &iterator, NULL, &value): %s (%d)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : -1));

	status = papiAttributeListAddBoolean(&list, PAPI_ATTR_REPLACE,
					"boolean-1", PAPI_TRUE);
	printf("papiAttributeListAddBoolean(&list, PAPI_ATTR_REPLACE, \"boolean-1\", PAPI_TRUE): %s\n", papiStatusString(status));

	status = papiAttributeListAddBoolean(&list, PAPI_ATTR_REPLACE,
					"boolean-2", PAPI_TRUE);
	printf("papiAttributeListAddBoolean(&list, PAPI_ATTR_REPLACE, \"boolean-2\", PAPI_TRUE): %s\n", papiStatusString(status));

	status = papiAttributeListGetBoolean(list, &iterator, "boolean-2",
					&value);
	printf("papiAttributeListGetBoolean(list, &iterator, \"boolean-2\", &value): %s (%d)\n",  
 		papiStatusString(status), (status == PAPI_OK ? value : -1));

	status = papiAttributeListAddBoolean(NULL, PAPI_ATTR_EXCL,
					"boolean-3", PAPI_TRUE);
	printf("papiAttributeListAddBoolean(NULL, PAPI_ATTR_EXCL, \"boolean-3\", PAPI_TRUE): %s\n", papiStatusString(status));

	status = papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL,
					NULL, PAPI_TRUE);
	printf("papiAttributeListAddBoolean(&list, PAPI_ATTR_EXCL, NULL, PAPI_TRUE): %s\n", papiStatusString(status));

	papiAttributeListFree(list);
}

test_datetime()
{
	papi_status_t status;
	papi_attribute_t **list = NULL;
	void *iterator = NULL;
	time_t value = NULL;
	char buf[64];

	status = papiAttributeListAddDatetime(&list, PAPI_ATTR_EXCL,
					"datetime-1", 0);
	printf("papiAttributeListAddDatetime(&list, PAPI_ATTR_EXCL, \"datetime-1\", 0): %s\n", papiStatusString(status));

	status = papiAttributeListAddDatetime(&list, PAPI_ATTR_EXCL,
					"datetime-1", 0);
	printf("papiAttributeListAddDatetime(&list, PAPI_ATTR_EXCL, \"datetime-1\", 0): %s\n", papiStatusString(status));

	status = papiAttributeListAddDatetime(&list, PAPI_ATTR_APPEND,
					"datetime-1", 0x80000000);
	printf("papiAttributeListAddDatetime(&list, PAPI_ATTR_APPEND, \"datetime-1\", 0x80000000): %s\n", papiStatusString(status));

	status = papiAttributeListGetDatetime(list, &iterator, "datetime-1",
					&value);
	timet_strftime(buf, sizeof (buf), "%X %x", &value);
	printf("papiAttributeListGetDatetime(list, &iterator, \"datetime-1\", &value): %s (%s)\n",  
 		papiStatusString(status), buf);

	status = papiAttributeListGetDatetime(list, &iterator, NULL, &value);
	timet_strftime(buf, sizeof (buf), "%X %x", &value);
	printf("papiAttributeListGetDatetime(list, &iterator, NULL, &value): %s (%s)\n",  
 		papiStatusString(status), buf);

	status = papiAttributeListAddDatetime(&list, PAPI_ATTR_REPLACE,
					"datetime-1", 0);
	printf("papiAttributeListAddDatetime(&list, PAPI_ATTR_REPLACE, \"datetime-1\", 0): %s\n", papiStatusString(status));

	status = papiAttributeListAddDatetime(&list, PAPI_ATTR_REPLACE,
					"datetime-2", 0);
	printf("papiAttributeListAddDatetime(&list, PAPI_ATTR_REPLACE, \"datetime-2\", 0): %s\n", papiStatusString(status));

	status = papiAttributeListGetDatetime(list, &iterator, "datetime-2",
					&value);
	timet_strftime(buf, sizeof (buf), "%X %x", &value);
	printf("papiAttributeListGetDatetime(list, &iterator, \"datetime-2\", &value): %s (%s)\n",  
 		papiStatusString(status), buf);

	status = papiAttributeListAddDatetime(NULL, PAPI_ATTR_EXCL,
					"datetime-3", 0);
	printf("papiAttributeListAddDatetime(NULL, PAPI_ATTR_EXCL, \"datetime-3\", 0): %s\n", papiStatusString(status));

	status = papiAttributeListAddDatetime(&list, PAPI_ATTR_EXCL,
					NULL, 0);
	printf("papiAttributeListAddDatetime(&list, PAPI_ATTR_EXCL, NULL, 0): %s\n", papiStatusString(status));

	papiAttributeListFree(list);
}


test_range()
{
	papi_status_t status;
	papi_attribute_t **list = NULL;
	void *iterator = NULL;
	int min, max;

	status = papiAttributeListAddRange(&list, PAPI_ATTR_EXCL,
					"range-1", 1, 2);
	printf("papiAttributeListAddRange(&list, PAPI_ATTR_EXCL, \"range-1\", 1, 2): %s\n", papiStatusString(status));

	status = papiAttributeListAddRange(&list, PAPI_ATTR_EXCL,
					"range-1", 2, 3);
	printf("papiAttributeListAddRange(&list, PAPI_ATTR_EXCL, \"range-1\", 2, 3): %s\n", papiStatusString(status));

	status = papiAttributeListAddRange(&list, PAPI_ATTR_APPEND,
					"range-1", 4, 9);
	printf("papiAttributeListAddRange(&list, PAPI_ATTR_APPEND, \"range-1\", 4, 9): %s\n", papiStatusString(status));

	status = papiAttributeListGetRange(list, &iterator, "range-1",
					&min, &max);
	printf("papiAttributeListGetRange(list, &iterator, \"range-1\", &min, &max): %s (%d - %d)\n",  
 		papiStatusString(status), min, max);

	status = papiAttributeListGetRange(list, &iterator, NULL, &min, &max);
	printf("papiAttributeListGetRange(list, &iterator, NULL, &min, &max): %s (%d - %d)\n",  
 		papiStatusString(status), min, max);

	status = papiAttributeListAddRange(&list, PAPI_ATTR_REPLACE,
					"range-1", 12, 555);
	printf("papiAttributeListAddRange(&list, PAPI_ATTR_REPLACE, \"range-1\", 12, 555): %s\n", papiStatusString(status));

	status = papiAttributeListAddRange(&list, PAPI_ATTR_REPLACE,
					"range-2", 42, 555);
	printf("papiAttributeListAddRange(&list, PAPI_ATTR_REPLACE, \"range-2\", 42, 555): %s\n", papiStatusString(status));

	status = papiAttributeListGetRange(list, &iterator, "range-2",
					&min, &max);
	printf("papiAttributeListGetRange(list, &iterator, \"range-2\", &min, &max): %s (%d - %d)\n",  
 		papiStatusString(status), min, max);

	status = papiAttributeListAddRange(NULL, PAPI_ATTR_EXCL,
					"range-3", 632, 3245);
	printf("papiAttributeListAddRange(NULL, PAPI_ATTR_EXCL, \"range-3\", 632, 3245): %s\n", papiStatusString(status));

	status = papiAttributeListAddRange(&list, PAPI_ATTR_EXCL,
					NULL, 632, 3245);
	printf("papiAttributeListAddRange(&list, PAPI_ATTR_EXCL, NULL, 632, 3245): %s\n", papiStatusString(status));

	status = papiAttributeListAddRange(&list, PAPI_ATTR_EXCL,
					"bad", 11632, 3245);
	printf("papiAttributeListAddRange(&list, PAPI_ATTR_EXCL, \"bad\", 11632, 3245): %s\n", papiStatusString(status));

	papiAttributeListFree(list);
}

test_resolution()
{
	papi_status_t status;
	papi_attribute_t **list = NULL;
	void *iterator = NULL;
	int x, y;
	papi_resolution_unit_t res;

	status = papiAttributeListAddResolution(&list, PAPI_ATTR_EXCL,
					"resolution-1", 1, 2, PAPI_RES_PER_INCH);
	printf("papiAttributeListAddResolution(&list, PAPI_ATTR_EXCL, \"resolution-1\", 1, 2, PAPI_RES_PER_INCH): %s\n", papiStatusString(status));

	status = papiAttributeListAddResolution(&list, PAPI_ATTR_EXCL,
					"resolution-1", 2, 3, PAPI_RES_PER_CM);
	printf("papiAttributeListAddResolution(&list, PAPI_ATTR_EXCL, \"resolution-1\", 2, 3, PAPI_RES_PER_CM): %s\n", papiStatusString(status));

	status = papiAttributeListAddResolution(&list, PAPI_ATTR_APPEND,
					"resolution-1", 4, 9, PAPI_RES_PER_CM);
	printf("papiAttributeListAddResolution(&list, PAPI_ATTR_APPEND, \"resolution-1\", 4, 9, PAPI_RES_PER_CM): %s\n", papiStatusString(status));

	status = papiAttributeListGetResolution(list, &iterator, "resolution-1",
					&x, &y, &res);
	printf("papiAttributeListGetResolution(list, &iterator, \"resolution-1\", &x, &y, &res): %s (%dx%d %d)\n",  
 		papiStatusString(status), x, y, res);

	status = papiAttributeListGetResolution(list, &iterator, NULL, &x, &y, &res);
	printf("papiAttributeListGetResolution(list, &iterator, NULL, &x, &y, &res): %s (%dx%d %d)\n",  
 		papiStatusString(status), x, y, res);

	status = papiAttributeListAddResolution(&list, PAPI_ATTR_REPLACE,
					"resolution-1", 12, 555, PAPI_RES_PER_CM);
	printf("papiAttributeListAddResolution(&list, PAPI_ATTR_REPLACE, \"resolution-1\", 12, 555, PAPI_RES_PER_CM): %s\n", papiStatusString(status));

	status = papiAttributeListAddResolution(&list, PAPI_ATTR_REPLACE,
					"resolution-2", 42, 555, PAPI_RES_PER_CM);
	printf("papiAttributeListAddResolution(&list, PAPI_ATTR_REPLACE, \"resolution-2\", 42, 555, PAPI_RES_PER_CM): %s\n", papiStatusString(status));

	status = papiAttributeListGetResolution(list, &iterator, "resolution-2",
					&x, &y, &res);
	printf("papiAttributeListGetResolution(list, &iterator, \"resolution-2\", &x, &y, &res): %s (%dx%d %d)\n",  
 		papiStatusString(status), x, y, res);

	status = papiAttributeListAddResolution(NULL, PAPI_ATTR_EXCL,
					"resolution-3", 632, 3245, PAPI_RES_PER_CM);
	printf("papiAttributeListAddResolution(NULL, PAPI_ATTR_EXCL, \"resolution-3\", 632, 3245, PAPI_RES_PER_CM): %s\n", papiStatusString(status));

	status = papiAttributeListAddResolution(&list, PAPI_ATTR_EXCL,
					NULL, 632, 3245, PAPI_RES_PER_CM);
	printf("papiAttributeListAddResolution(&list, PAPI_ATTR_EXCL, NULL, 632, 3245, PAPI_RES_PER_CM): %s\n", papiStatusString(status));

	status = papiAttributeListAddResolution(&list, PAPI_ATTR_EXCL,
					"bad", 11632, 3245, PAPI_RES_PER_CM);
	printf("papiAttributeListAddResolution(&list, PAPI_ATTR_EXCL, \"bad\", 11632, 3245, PAPI_RES_PER_CM): %s\n", papiStatusString(status));

	papiAttributeListFree(list);
}


/*
 * This is hardly an exaustive test of the attribute interface, but it performs
 * some basic tests to verify that it is minimally working.
 */
main(int ac, char *av[])
{
	test_string();
	test_integer();
	test_boolean();
	test_range();
	test_resolution();
	test_datetime();
	test_collection();
}
